#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import threading

import codecs
import urllib2

import ast

from bs4 import BeautifulSoup as Soup
try:
    import simplejson as json
except:
    import json

import sip
sip.setapi('QVariant', 2)

from PyQt4 import QtGui, QtCore, uic
from PyQt4.QtCore import Qt, QMetaObject, QThread, pyqtSlot, Q_ARG, QLocale
from PyQt4.QtGui import QDialog, QFileDialog, QColor

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

from osm import OSM


SCRIPTNAME = ''
try:
    SCRIPTNAME = __file__
except:
    SCRIPTNAME = sys.argv[0]

CWD = os.path.abspath(os.path.dirname(SCRIPTNAME))

def isBool(v):
  return v.lower() in ("yes", "true", "t", "1")

class OutLog:
    log = None
    myWindow = None

    def __init__(self, edit, myWindow, out=None, color=None):
        """(edit, out=None, color=None) -> can write stdout, stderr to a
        QTextEdit.
        edit = QTextEdit
        out = alternate stream ( can be the original sys.stdout )
        color = alternate color (i.e. color stderr a different color)
        """
        self.edit = edit
        self.out = None
        self.color = color

        self.myWindow = myWindow

        # log = LOG(self.myWindow)

    def write(self, m):
        if self.color:
            tc = self.edit.textColor()
            # self.edit.setTextColor(self.color)
            QMetaObject.invokeMethod(self.myWindow, "updateOutputColor", Qt.QueuedConnection, Q_ARG("QColor", self.color))

        # try:
        #     self.edit.moveCursor(QtGui.QTextCursor.End)
        #     self.edit.insertPlainText( m )
        # except:
        #     self.log.run(m)
        QMetaObject.invokeMethod(self.myWindow, "updateOutputText", Qt.QueuedConnection, Q_ARG("QString", m))


        if self.color:
            # self.edit.setTextColor(tc)
            QMetaObject.invokeMethod(self.myWindow, "updateOutputColor", Qt.QueuedConnection, Q_ARG("QColor", tc))

        if self.out:
            self.out.write(m)

        pass


class MapWindow(QtGui.QMainWindow):
    ui_main = None
    UIFILE_MAIN = 'mapmaker.ui'
    IniFile = os.path.splitext(SCRIPTNAME)[0] + '.ini'
    MapListFile = os.path.splitext(SCRIPTNAME)[0] + '.loc'
    MapList = dict()

    count = 0
    lock = threading.Lock()

    osm = None

    def __init__(self):
        QtGui.QMainWindow.__init__(self)

        ui_file = os.path.join(CWD, self.UIFILE_MAIN)
        if os.path.isfile(ui_file):
            # Set up the user interface from Designer.
            # self.setupUi(self)

            self.ui_main = uic.loadUi(ui_file)
            self.ui_main.show()

            # Make some local modifications.

            # Connect up the buttons.
            QtCore.QObject.connect(self.ui_main, QtCore.SIGNAL('rejected()'), self.on_btnBox_rejected)
            QtCore.QObject.connect(self.ui_main.btnBox, QtCore.SIGNAL('rejected()'), self.on_btnBox_rejected)

            QtCore.QObject.connect(self.ui_main.btnAddToMapUsed, QtCore.SIGNAL('clicked()'), self.on_btnAddToMapUsed_clicked)
            QtCore.QObject.connect(self.ui_main.cbbMapUsed, QtCore.SIGNAL('currentIndexChanged(QString)'), self.on_cbbMapUsed_currentIndexChanged)

            QtCore.QObject.connect(self.ui_main.btnContours, QtCore.SIGNAL('clicked()'), self.on_btnContours_clicked)
            QtCore.QObject.connect(self.ui_main.btnMakeMap, QtCore.SIGNAL('clicked()'), self.on_btnMakeMap_clicked)

            QtCore.QObject.connect(self.ui_main.btnBrowsePolygon, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseHGT, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseContours, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseJAVA, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseOSMOSIS, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseOSMConvert, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseMkgmap, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)
            QtCore.QObject.connect(self.ui_main.btnBrowseTarget, QtCore.SIGNAL('clicked()'), self.on_FolderSelector_clicked)

            # Init the path
            self.InitSet()

            # Load last setting
            self.MapListLoad()
            self.IniLoad()

            self.ui_main.tabMain.setCurrentIndex(1)
            # redir stdout, stderr to GUI
            # sys.stdout = OutLog(self.ui_main.edOutput, self, sys.stdout)
            # sys.stderr = OutLog(self.ui_main.edOutput, self, sys.stderr, QtGui.QColor(255,0,0) )

            pass

    def getOptions(self):
        values = dict()

        values['Polygon'] = unicode(self.ui_main.edSetPolygon.text())
        values['HGT'] = unicode(self.ui_main.edSetHGT.text())
        values['Contours'] = unicode(self.ui_main.edSetContours.text())
        values['JAVAHOME'] = unicode(self.ui_main.edSetJAVAHome.text())
        values['OSMOSIS'] = unicode(self.ui_main.edSetOSMOSIS.text())
        values['OSMConvert'] = unicode(self.ui_main.edSetOSMConvert.text())
        values['mkgmap'] = unicode(self.ui_main.edSetMKGMAP.text())
        values['MapFolder'] = unicode(self.ui_main.edSetMapFolder.text())

        values['minLat'] = self.ui_main.edMapLatMin.value()
        values['minLon'] = self.ui_main.edMapLonMin.value()
        values['maxLat'] = self.ui_main.edMapLatMax.value()
        values['maxLon'] = self.ui_main.edMapLonMax.value()

        values['lastMapName'] = unicode(self.ui_main.edMapName.text())

        localeValue = 'en'
        if self.ui_main.chkSetLocale_en.isChecked():
            localeValue = 'en'
        elif self.ui_main.chkSetLocale_zh_CN.isChecked():
            localeValue = 'zh_CN'
        elif self.ui_main.chkSetLocale_zh_TW.isChecked():
            localeValue = 'zh_TW'
        values['locale'] = localeValue

        BBOX = dict()
        BBOX['min'] = dict({'lat':values['minLat'], 'lon':values['minLon']})
        BBOX['max'] = dict({'lat':values['maxLat'], 'lon':values['maxLon']})

        options = dict()
        options['download'] = self.ui_main.cbOptDownload.isChecked()
        options['tag'] = self.ui_main.cbOptTag.isChecked()
        options['cut'] = self.ui_main.cbOptCut.isChecked()
        options['sort'] = self.ui_main.cbOptSort.isChecked()
        options['merge'] = self.ui_main.cbOptMerge.isChecked()
        options['convert'] = self.ui_main.cbOptBinary.isChecked()
        options['hd'] = self.ui_main.cbOptHD.isChecked()

        maptype = dict()
        maptype['garmin'] = self.ui_main.cbMapGarmin.isChecked()
        maptype['mapforge'] = self.ui_main.cbMapForge.isChecked()

        options['maptype'] = maptype

        return values['lastMapName'], BBOX, values, options
        pass

    @pyqtSlot(int)
    def updateProgress(self, value):
        self.ui_main.progressBar.setValue(value)

        pass

    @pyqtSlot(unicode)
    def updateOutputText(self, text):
        self.ui_main.edOutput.moveCursor(QtGui.QTextCursor.End)
        self.ui_main.edOutput.insertPlainText( text )

        pass

    @pyqtSlot(QColor)
    def updateOutputColor(self, color):
        self.ui_main.edOutput.setTextColor(color)

        pass

    @pyqtSlot()
    def on_FolderSelector_clicked(self):
        sender = self.sender()
        folder_old = CWD

        if sender is self.ui_main.btnBrowsePolygon:
            folder_old = self.ui_main.edSetPolygon.text()
        elif sender is self.ui_main.btnBrowseHGT:
            folder_old = self.ui_main.edSetHGT.text()
        elif sender is self.ui_main.btnBrowseContours:
            folder_old = self.ui_main.edSetContours.text()
        elif sender is self.ui_main.btnBrowseJAVA:
            folder_old = self.ui_main.edSetJAVAHome.text()
        elif sender is self.ui_main.btnBrowseOSMOSIS:
            folder_old = self.ui_main.edSetOSMOSIS.text()
        elif sender is self.ui_main.btnBrowseOSMConvert:
            folder_old = self.ui_main.edSetOSMConvert.text()
        elif sender is self.ui_main.btnBrowseMkgmap:
            folder_old = self.ui_main.edSetMKGMAP.text()
        elif sender is self.ui_main.btnBrowseTarget:
            folder_old = self.ui_main.edSetMapFolder.text()

        if not os.path.exists(folder_old):
            folder_new = CWD
        folder_new = unicode(QFileDialog.getExistingDirectory(self, self.tr(u"Select Directory"), folder_old))
        print(folder_new)

        if folder_new:
            if sender is self.ui_main.btnBrowsePolygon:
                self.ui_main.edSetPolygon.setText(folder_new)
            elif sender is self.ui_main.btnBrowseHGT:
                self.ui_main.edSetHGT.setText(folder_new)
            elif sender is self.ui_main.btnBrowseContours:
                self.ui_main.edSetContours.setText(folder_new)
            elif sender is self.ui_main.btnBrowseJAVA:
                self.ui_main.edSetJAVAHome.setText(folder_new)
            elif sender is self.ui_main.btnBrowseOSMOSIS:
                self.ui_main.edSetOSMOSIS.setText(folder_new)
            elif sender is self.ui_main.btnBrowseOSMConvert:
                self.ui_main.edSetOSMConvert.setText(folder_new)
            elif sender is self.ui_main.btnBrowseMkgmap:
                self.ui_main.edSetMKGMAP.setText(folder_new)
            elif sender is self.ui_main.btnBrowseTarget:
                self.ui_main.edSetMapFolder.setText(folder_new)

        pass

    @pyqtSlot()
    def on_btnContours_clicked(self):
        mapname, bbox, values, options = self.getOptions()
        self.osm = OSM(mapname, bbox, values, options)
        # self.osm.contours()
        threading.Thread(target=self.runContours, args=(), name='thread-makingcontours').start()
        pass

    @pyqtSlot()
    def on_btnMakeMap_clicked(self):
        mapname, bbox, values, options = self.getOptions()
        self.osm = OSM(mapname, bbox, values, options)

        threading.Thread(target=self.runMap, args=(), name='thread-makingmap').start()
        # self.osm.test()

        pass

    @pyqtSlot()
    def on_btnBox_rejected(self):
        self.IniSave()
        self.MapListSave()

        pass

    @pyqtSlot()
    def on_btnAddToMapUsed_clicked(self):

        Map = unicode(self.ui_main.edMapName.text())

        boundary = dict()
        boundary['minLat'] = unicode(self.ui_main.edMapLatMin.value())
        boundary['minLon'] = unicode(self.ui_main.edMapLonMin.value())
        boundary['maxLat'] = unicode(self.ui_main.edMapLatMax.value())
        boundary['maxLon'] = unicode(self.ui_main.edMapLonMax.value())
        self.MapList[Map] = boundary

        self.ui_main.cbbMapUsed.addItem(Map, boundary)
        pass

    @pyqtSlot(unicode)
    def on_cbbMapUsed_currentIndexChanged(self, text):
        name = unicode(text)
        if name in self.MapList:
            Map = self.MapList[name]

            self.ui_main.edMapName.setText(name)

            self.ui_main.edLatMin.setValue(Map['minLat'])
            self.ui_main.edLonMin.setValue(Map['minLon'])
            self.ui_main.edLatMax.setValue(Map['maxLat'])
            self.ui_main.edLonMax.setValue(Map['maxLon'])

            self.ui_main.edMapLatMin.setValue(Map['minLat'])
            self.ui_main.edMapLonMin.setValue(Map['minLon'])
            self.ui_main.edMapLatMax.setValue(Map['maxLat'])
            self.ui_main.edMapLonMax.setValue(Map['maxLon'])
        pass

    def InitSet(self):
        if self.ui_main:
            self.ui_main.edSetPolygon.setText(os.path.join(CWD, '_polygon'))
            self.ui_main.edSetHGT.setText(os.path.join(CWD, '_hgt'))
            self.ui_main.edSetContours.setText(os.path.join(CWD, '_srtm-data'))
            if 'JAVA_HOME' in os.environ:
                self.ui_main.edSetJAVAHome.setText(os.environ['JAVA_HOME'])
            else:
                self.ui_main.edSetJAVAHome.setText(os.path.join(CWD, 'jre7'))
            self.ui_main.edSetOSMOSIS.setText(os.path.join(CWD, 'OSMOSIS', 'bin', 'osmosis'))
            self.ui_main.edSetMKGMAP.setText(os.path.join(CWD, 'mkgmap'))
            self.ui_main.edSetOSMConvert.setText(os.path.join(CWD, 'osmconvert'))
            self.ui_main.edSetMapFolder.setText(os.path.join(CWD, '_maps'))

            # self.ui_main.cbOptDownload.setChecked(True)
            self.ui_main.cbMapForge.setChecked(True)
            self.ui_main.cbOptTag.setChecked(True)
            self.ui_main.cbOptBinary.setChecked(True)

        pass

    def IniLoad(self, useJSON=True):
        if os.path.isfile(self.IniFile):
            with codecs.open(self.IniFile, 'r', encoding='utf8') as f:
                if useJSON:
                    jData = f.readlines()
                    options = json.loads('\n'.join(jData))

                    values = options['setting']
                    if 'Polygon' in values:
                        self.ui_main.edSetPolygon.setText(values['Polygon'])
                    if 'HGT' in values:
                        self.ui_main.edSetHGT.setText(values['HGT'])
                    if 'Contours' in values:
                        self.ui_main.edSetContours.setText(values['Contours'])
                    if 'JAVAHOME' in values:
                        self.ui_main.edSetJAVAHome.setText(values['JAVAHOME'])
                    if 'OSMOSIS' in values:
                        self.ui_main.edSetOSMOSIS.setText(values['OSMOSIS'])
                    if 'mkgmap' in values:
                        self.ui_main.edSetMKGMAP.setText(values['mkgmap'])
                    if 'OSMConvert' in values:
                        self.ui_main.edSetOSMConvert.setText(values['OSMConvert'])
                    if 'MapFolder' in values:
                        self.ui_main.edSetMapFolder.setText(values['MapFolder'])

                    if 'locale' in values:
                        localevalue = values['locale']
                        if localevalue == 'en':
                            self.ui_main.chkSetLocale_en.setChecked(True)
                        elif localevalue == 'zh_CN':
                            self.ui_main.chkSetLocale_zh_CN.setChecked(True)
                        elif localevalue == 'zh_TW':
                            self.ui_main.chkSetLocale_zh_TW.setChecked(True)

                    if 'minLat' in values:
                        self.ui_main.edMapLatMin.setValue(float(values['minLat']))
                    if 'minLon' in values:
                        self.ui_main.edMapLonMin.setValue(float(values['minLon']))
                    if 'maxLat' in values:
                        self.ui_main.edMapLatMax.setValue(float(values['maxLat']))
                    if 'maxLon' in values:
                        self.ui_main.edMapLonMax.setValue(float(values['maxLon']))

                    if 'minLat' in values:
                        self.ui_main.edLatMin.setValue(float(values['minLat']))
                    if 'minLon' in values:
                        self.ui_main.edLonMin.setValue(float(values['minLon']))
                    if 'maxLat' in values:
                        self.ui_main.edLatMax.setValue(float(values['maxLat']))
                    if 'maxLon' in values:
                        self.ui_main.edLonMax.setValue(float(values['maxLon']))

                    if 'lastMapName' in values:
                        name = values['lastMapName']
                        self.ui_main.edMapName.setText(name)
                        if name in self.MapList:
                            idx = self.ui_main.cbbMapUsed.findText(name)
                            if idx >= 0:
                                self.ui_main.cbbMapUsed.setCurrentIndex(idx)

                    values = options['action']
                    if 'do_download' in values:
                        if values['do_download']:
                            self.ui_main.cbOptDownload.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptDownload.setCheckState(Qt.Unchecked)
                    if 'do_cut' in values:
                        if values['do_cut']:
                            self.ui_main.cbOptCut.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptCut.setCheckState(Qt.Unchecked)
                    if 'do_tag' in values:
                        if values['do_tag']:
                            self.ui_main.cbOptTag.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptTag.setCheckState(Qt.Unchecked)
                    if 'do_sort' in values:
                        if values['do_sort']:
                            self.ui_main.cbOptSort.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptSort.setCheckState(Qt.Unchecked)
                    if 'do_merge' in values:
                        if values['do_merge']:
                            self.ui_main.cbOptMerge.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptMerge.setCheckState(Qt.Unchecked)
                    if 'do_convert' in values:
                        if values['do_convert']:
                            self.ui_main.cbOptBinary.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptBinary.setCheckState(Qt.Unchecked)
                    if 'do_hd' in values:
                        if values['do_hd']:
                            self.ui_main.cbOptHD.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptHD.setCheckState(Qt.Unchecked)


                    if 'maptype' in values:
                        maptype = values['maptype']
                        if maptype['garmin']:
                            self.ui_main.cbMapGarmin.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbMapGarmin.setCheckState(Qt.Unchecked)

                        if maptype['mapforge']:
                            self.ui_main.cbMapForge.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbMapForge.setCheckState(Qt.Unchecked)

                else:
                    soup = Soup(f, ["lxml", "xml"])

                    values = dict()
                    for value in soup.find_all('setting'):
                        values[value['k']] = value['v']

                    if 'Polygon' in values:
                        self.ui_main.edSetPolygon.setText(values['Polygon'])
                    if 'HGT' in values:
                        self.ui_main.edSetHGT.setText(values['HGT'])
                    if 'Contours' in values:
                        self.ui_main.edSetContours.setText(values['Contours'])
                    if 'JAVAHOME' in values:
                        self.ui_main.edSetJAVAHome.setText(values['JAVAHOME'])
                    if 'OSMOSIS' in values:
                        self.ui_main.edSetOSMOSIS.setText(values['OSMOSIS'])
                    if 'mkgmap' in values:
                        self.ui_main.edSetMKGMAP.setText(values['mkgmap'])
                    if 'OSMConvert' in values:
                        self.ui_main.edSetOSMConvert.setText(values['OSMConvert'])
                    if 'MapFolder' in values:
                        self.ui_main.edSetMapFolder.setText(values['MapFolder'])

                    if 'locale' in values:
                        localevalue = values['locale']
                        if localevalue == 'en':
                            self.ui_main.chkSetLocale_en.setChecked(True)
                        elif localevalue == 'zh_CN':
                            self.ui_main.chkSetLocale_zh_CN.setChecked(True)
                        elif localevalue == 'zh_TW':
                            self.ui_main.chkSetLocale_zh_TW.setChecked(True)

                    if 'minLat' in values:
                        self.ui_main.edMapLatMin.setValue(float(values['minLat']))
                    if 'minLon' in values:
                        self.ui_main.edMapLonMin.setValue(float(values['minLon']))
                    if 'maxLat' in values:
                        self.ui_main.edMapLatMax.setValue(float(values['maxLat']))
                    if 'maxLon' in values:
                        self.ui_main.edMapLonMax.setValue(float(values['maxLon']))

                    if 'minLat' in values:
                        self.ui_main.edLatMin.setValue(float(values['minLat']))
                    if 'minLon' in values:
                        self.ui_main.edLonMin.setValue(float(values['minLon']))
                    if 'maxLat' in values:
                        self.ui_main.edLatMax.setValue(float(values['maxLat']))
                    if 'maxLon' in values:
                        self.ui_main.edLonMax.setValue(float(values['maxLon']))

                    if 'lastMapName' in values:
                        name = values['lastMapName']
                        self.ui_main.edMapName.setText(name)
                        if name in self.MapList:
                            idx = self.ui_main.cbbMapUsed.findText(name)
                            if idx >= 0:
                                self.ui_main.cbbMapUsed.setCurrentIndex(idx)

                    values = dict()
                    for value in soup.find_all('action'):
                        values[value['k']] = value['v']

                    if 'do_download' in values:
                        if isBool(values['do_download']):
                            self.ui_main.cbOptDownload.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptDownload.setCheckState(Qt.Unchecked)
                    if 'do_cut' in values:
                        if isBool(values['do_cut']):
                            self.ui_main.cbOptCut.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptCut.setCheckState(Qt.Unchecked)
                    if 'do_tag' in values:
                        if isBool(values['do_tag']):
                            self.ui_main.cbOptTag.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptTag.setCheckState(Qt.Unchecked)
                    if 'do_sort' in values:
                        if isBool(values['do_sort']):
                            self.ui_main.cbOptSort.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptSort.setCheckState(Qt.Unchecked)
                    if 'do_merge' in values:
                        if isBool(values['do_merge']):
                            self.ui_main.cbOptMerge.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptMerge.setCheckState(Qt.Unchecked)
                    if 'do_convert' in values:
                        if isBool(values['do_convert']):
                            self.ui_main.cbOptBinary.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptBinary.setCheckState(Qt.Unchecked)
                    if 'do_hd' in values:
                        if isBool(values['do_hd']):
                            self.ui_main.cbOptHD.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbOptHD.setCheckState(Qt.Unchecked)


                    if 'maptype' in values:
                        maptype = ast.literal_eval(values['maptype'])
                        if maptype['garmin']:
                            self.ui_main.cbMapGarmin.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbMapGarmin.setCheckState(Qt.Unchecked)

                        if maptype['mapforge']:
                            self.ui_main.cbMapForge.setCheckState(Qt.Checked)
                        else:
                            self.ui_main.cbMapForge.setCheckState(Qt.Unchecked)

        pass

    def IniSave(self, useJSON=True):
        with codecs.open(self.IniFile, 'w', encoding='utf8') as f:
            if useJSON:
                options = dict()

                values = dict()

                values['Polygon'] = unicode(self.ui_main.edSetPolygon.text())
                values['HGT'] = unicode(self.ui_main.edSetHGT.text())
                values['Contours'] = unicode(self.ui_main.edSetContours.text())
                values['JAVAHOME'] = unicode(self.ui_main.edSetJAVAHome.text())
                values['OSMOSIS'] = unicode(self.ui_main.edSetOSMOSIS.text())
                values['mkgmap'] = unicode(self.ui_main.edSetMKGMAP.text())
                values['OSMConvert'] = unicode(self.ui_main.edSetOSMConvert.text())
                values['MapFolder'] = unicode(self.ui_main.edSetMapFolder.text())

                localeValue = 'en'
                if self.ui_main.chkSetLocale_en.isChecked():
                    localeValue = 'en'
                elif self.ui_main.chkSetLocale_zh_CN.isChecked():
                    localeValue = 'zh_CN'
                elif self.ui_main.chkSetLocale_zh_TW.isChecked():
                    localeValue = 'zh_TW'
                values['locale'] = localeValue

                values['minLat'] = self.ui_main.edMapLatMin.value()
                values['minLon'] = self.ui_main.edMapLonMin.value()
                values['maxLat'] = self.ui_main.edMapLatMax.value()
                values['maxLon'] = self.ui_main.edMapLonMax.value()

                values['lastMapName'] = unicode(self.ui_main.edMapName.text())

                options['setting'] = values

                values = dict()
                values['do_download'] = self.ui_main.cbOptDownload.isChecked()
                values['do_cut'] = self.ui_main.cbOptCut.isChecked()
                values['do_tag'] = self.ui_main.cbOptTag.isChecked()
                values['do_sort'] = self.ui_main.cbOptSort.isChecked()
                values['do_merge'] = self.ui_main.cbOptMerge.isChecked()
                values['do_convert'] = self.ui_main.cbOptBinary.isChecked()
                values['do_hd'] = self.ui_main.cbOptHD.isChecked()

                maptype = dict()
                maptype['garmin'] = self.ui_main.cbMapGarmin.isChecked()
                maptype['mapforge'] = self.ui_main.cbMapForge.isChecked()

                values['maptype'] = maptype

                options['action'] = values


                lines = json.dumps(options, indent=2, sort_keys=True)
                # json.dump(f, options, indent=2)

            else:
                soup = Soup('', ["lxml", "xml"])
                soup.ROOT_TAG_NAME = 'MapForgeMaker'

                root = soup.new_tag(soup.ROOT_TAG_NAME)
                soup.append(root)

                settings = soup.new_tag('Settings')
                root.append(settings)

                values = dict()
                values['Polygon'] = self.ui_main.edSetPolygon.text()
                values['HGT'] = self.ui_main.edSetHGT.text()
                values['Contours'] = self.ui_main.edSetContours.text()
                values['JAVAHOME'] = self.ui_main.edSetJAVAHome.text()
                values['OSMOSIS'] = self.ui_main.edSetOSMOSIS.text()
                values['mkgmap'] = self.ui_main.edSetMKGMAP.text()
                values['OSMConvert'] = self.ui_main.edSetOSMConvert.text()
                values['MapFolder'] = self.ui_main.edSetMapFolder.text()

                localeValue = 'en'
                if self.ui_main.chkSetLocale_en.isChecked():
                    localeValue = 'en'
                elif self.ui_main.chkSetLocale_zh_CN.isChecked():
                    localeValue = 'zh_CN'
                elif self.ui_main.chkSetLocale_zh_TW.isChecked():
                    localeValue = 'zh_TW'
                values['locale'] = localeValue

                values['minLat'] = self.ui_main.edMapLatMin.value()
                values['minLon'] = self.ui_main.edMapLonMin.value()
                values['maxLat'] = self.ui_main.edMapLatMax.value()
                values['maxLon'] = self.ui_main.edMapLonMax.value()

                values['lastMapName'] = self.ui_main.edMapName.text()

                for k in values:
                    v = values[k]
                    setting = soup.new_tag('setting')
                    setting['k'] = k
                    setting['v'] = v
                    settings.append(setting)

                actions = soup.new_tag('Actions')
                root.append(actions)

                values = dict()
                values['do_download'] = self.ui_main.cbOptDownload.isChecked()
                values['do_cut'] = self.ui_main.cbOptCut.isChecked()
                values['do_tag'] = self.ui_main.cbOptTag.isChecked()
                values['do_sort'] = self.ui_main.cbOptSort.isChecked()
                values['do_merge'] = self.ui_main.cbOptMerge.isChecked()
                values['do_convert'] = self.ui_main.cbOptBinary.isChecked()
                values['do_hd'] = self.ui_main.cbOptHD.isChecked()


                maptype = dict()
                maptype['garmin'] = self.ui_main.cbMapGarmin.isChecked()
                maptype['mapforge'] = self.ui_main.cbMapForge.isChecked()

                values['maptype'] = maptype


                for k in values:
                    v = values[k]
                    action = soup.new_tag('action')
                    action['k'] = k
                    action['v'] = v
                    actions.append(action)

                lines = soup.prettify()
                # print(lines)

            f.write(lines)

        pass

    def MapListLoad(self, useJSON=True):
        if os.path.isfile(self.MapListFile):
            with codecs.open(self.MapListFile, 'r', encoding='utf8') as f:
                if useJSON:
                    jData = f.readlines()
                    options = json.loads('\n'.join(jData))

                    self.ui_main.cbbMapUsed.clear()
                    for Map in options:
                        boundary = options[Map]
                        self.MapList[Map] = boundary

                        self.ui_main.cbbMapUsed.addItem(Map, boundary)
                else:
                    soup = Soup(f, ["lxml", "xml"])

                    self.ui_main.cbbMapUsed.clear()
                    for Map in soup.find_all('Map'):
                        boundary = dict()
                        boundary['minLat'] = float(Map['minLat'])
                        boundary['minLon'] = float(Map['minLon'])
                        boundary['maxLat'] = float(Map['maxLat'])
                        boundary['maxLon'] = float(Map['maxLon'])
                        self.MapList[Map['name']] = boundary

                        self.ui_main.cbbMapUsed.addItem(Map['name'], boundary)
        pass

    def MapListSave(self, useJSON=True):
        with codecs.open(self.MapListFile, 'w', encoding='utf8') as f:
            if useJSON:
                options = dict()
                for k in self.MapList:
                    Map = dict()
                    Map['name'] = k
                    Map['minLat'] = self.MapList[k]['minLat']
                    Map['minLon'] = self.MapList[k]['minLon']
                    Map['maxLat'] = self.MapList[k]['maxLat']
                    Map['maxLon'] = self.MapList[k]['maxLon']

                    options[k] = Map

                lines = json.dumps(options, indent=2, sort_keys=True)
            else:
                soup = Soup('', ["lxml", "xml"])
                soup.ROOT_TAG_NAME = 'MapList'

                root = soup.new_tag(soup.ROOT_TAG_NAME)
                soup.append(root)

                maps = soup.new_tag('Maps')
                root.append(maps)

                for k in self.MapList:
                    Map = soup.new_tag('Map')
                    Map['name'] = k
                    Map['minLat'] = self.MapList[k]['minLat']
                    Map['minLon'] = self.MapList[k]['minLon']
                    Map['maxLat'] = self.MapList[k]['maxLat']
                    Map['maxLon'] = self.MapList[k]['maxLon']
                    maps.append(Map)


                lines = soup.prettify()
            # print(lines)

            f.write(lines)
        pass

    def runMap(self):
        self.lock.acquire()

        binary = self.ui_main.cbOptBinary.isChecked()
        # self.osm.test()
        self.osm.MapMaking(binary)

        self.lock.release()

        pass

    def runContours(self):
        self.lock.acquire()

        self.osm.contours()

        self.lock.release()

        pass


if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    translator = QtCore.QTranslator()
    translator.load(QLocale.system(), os.path.splitext(os.path.basename(SCRIPTNAME))[0], prefix='_', directory = os.path.join(CWD, 'i18n'), suffix='.qm')
    app.installTranslator(translator)

    win = MapWindow()

    if sys.stdout.isatty():
        pass
    else:
        sys.stdout = OutLog(win.ui_main.edOutput, win, sys.stdout)
        sys.stderr = OutLog(win.ui_main.edOutput, win, sys.stderr, QtGui.QColor(255,0,0))

    sys.exit(app.exec_())
