#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import getopt

import math

import urllib

from osm_utils import *


class TERRAIN:
    def download(self):
        pass


class OSM:
    MAP = ''
    BBOX = dict()
    BBOX0 = ''
    BBOX1 = ''

    TARGET_FOLDER = ''

    LANG = 'zh'

    POLYGON = ''
    SRTM = ''
    HGT = ''

    JAVAHOME = ''

    OSMOSIS = ''
    OSMCONVERT = ''

    MKGMAP = ''
    SPLITTER = ''

    do_download = False
    do_tag = False
    do_cut = False
    do_sort = False
    do_merge = False
    do_convert = False
    do_hd = False

    do_maptype = dict()

    def __init__(self, mapname, bbox, settings, options):
        self.MAP = mapname

        self.BBOX = bbox
        self.BBOX0 = '%.2f,%.2f,%.2f,%.2f' % (bbox['min']['lon'],bbox['min']['lat'],bbox['max']['lon'],bbox['max']['lat'])
        self.BBOX1 = '%.2f,%.2f,%.2f,%.2f' % (bbox['min']['lat'],bbox['min']['lon'],bbox['max']['lat'],bbox['max']['lon'])

        self.POLYGON = settings['Polygon']
        self.SRTM = settings['Contours']
        self.HGT = settings['HGT']

        self.JAVAHOME = settings['JAVAHOME']

        self.OSMOSIS = os.path.join(settings['OSMOSIS'], 'osmosis')
        self.OSMCONVERT = os.path.join(settings['OSMConvert'], 'osmconvert')

        self.MKGMAP = os.path.join(settings['mkgmap'], 'mkgmap')
        self.SPLITTER = os.path.join(settings['mkgmap'], 'splitter')

        self.TARGET_FOLDER = settings['MapFolder']

        if 'locale' in settings:
            self.LANG = settings['locale'][:2]

        self.do_download = options['download']
        self.do_tag = options['tag']
        self.do_cut = options['cut']
        self.do_sort = options['sort']
        self.do_merge = options['merge']
        self.do_convert = options['convert']
        self.do_hd = options['hd']

        self.do_maptype = options['maptype']

        pass

    def bboxSplit(self):
        bboxs = []

        minlon = self.BBOX['min']['lon']
        minlat = self.BBOX['min']['lat']
        maxlon = self.BBOX['max']['lon']
        maxlat = self.BBOX['max']['lat']

        minlon_I = int(minlon)+1
        minlat_I = int(minlat)+1

        maxlon_I = int(maxlon)
        maxlat_I = int(maxlat)

        l = []
        l.append((minlon, minlat))
        for x in range(minlon_I, maxlon_I+1):
            l.append((x, minlat))
        l.append((maxlon, minlat))
        bboxs.append(l)

        for y in range(minlat_I, maxlat_I+1):
            l = []
            l.append((minlon, y))
            for x in range(minlon_I, maxlon_I+1):
                l.append((x, y))
            l.append((maxlon, y))
            bboxs.append(l)

        l = []
        l.append((minlon, maxlat))
        for x in range(minlon_I, maxlon_I+1):
            l.append((x, maxlat))
        l.append((maxlon, maxlat))
        bboxs.append(l)

        # print(bboxs)

        bboxlist = []
        for idx_Y in xrange(len(bboxs)-1):
            for idx_X in xrange(len(bboxs[idx_Y])-1):
                point_s = bboxs[idx_Y][idx_X]
                point_e = bboxs[idx_Y+1][idx_X+1]

                lon_s = point_s[0]
                lat_s = point_s[1]

                lon_e = point_e[0]
                lat_e = point_e[1]

                if lon_s < 0:
                    fn_lon = 'W%3d' % int(lon_s)
                else:
                    fn_lon = 'E%3d' % int(lon_s)

                if lat_s < 0:
                    fn_lat = 'S%2d' % int(lat_s)
                else:
                    fn_lat = 'N%2d' % int(lat_s)

                fname = '%s%s' % (fn_lon, fn_lat)
                farea =  '%.2f,%.2f,%.2f,%.2f' % (lon_s,lat_s,lon_e,lat_e)

                item = dict()
                item['file'] = fname
                item['bbox'] = farea

                bboxlist.append(item)

        return(bboxlist)

        pass

    def download_list(self, binary=True):
        print('Downloading %s.osm from overpass-api.de' % self.MAP)
        print('===============================================')

        tiles = self.bboxSplit()
        # print tiles
        # return

        if binary:
            exto = '.pbf'
        else:
            exto = '.osm'
        fout = os.path.join(self.TARGET_FOLDER, self.MAP+exto)
        # print(fout)

        if not os.path.exists(self.TARGET_FOLDER):
            os.makedirs(self.TARGET_FOLDER)

        url = 'http://www.overpass-api.de/api/xapi_meta?*[bbox=%s]'
        ftarget = os.path.join(self.TARGET_FOLDER, '_cache')

        for tile in tiles:
            url_bbox = url % tile['bbox']
            furl = os.path.join(ftarget, tile['file']+'.osm')
            urllib.urlretrieve(url_bbox, furl)
            print('%s downloaded.' % furl)

        options = []
        for idx in xrange(len(tiles)):
            url_bbox = url % tiles[idx]['bbox']
            furl = os.path.join(ftarget, tiles[idx]['file'])
            if idx == 0:
                options.append('%s "%s.osm"' % (self.OSMCONVERT, furl))
            else:
                options.append('%s - "%s.osm"' % (self.OSMCONVERT, furl))

        # cmdline = '%s --out-pbf -o="%s"' % (' | '.join(options), fout)
        if binary:
            of = '--out-pbf'
        else:
            of = '--out-osm'
        cmdline = '%s %s -o="%s"' % (' | '.join(options), of, fout)

        # print(cmdline)
        ret = run(cmdline)

        # ret = (None, None, None)

        print('-----------------------------------------------')
        print('')

        if ret[0] != 0:
            fout = None

        return fout
        pass

    def contours(self):
        print('Downloading %s SRTM/DEM data from NASA and making contours osm file' % self.SRTM)
        print('===============================================')

        if not os.path.exists(self.SRTM):
            os.makedirs(self.SRTM)
        fout = os.path.join(self.SRTM, self.MAP)

        options = []
        options.append('--polygon "%s"' % os.path.join(self.POLYGON, self.MAP+'.poly'))
        options.append('-o "%s"' % fout)
        options.append('--source %s' % 'view1,view3,srtm1,srtm3')
        options.append('-s %d' % 20)
        options.append('-c %d,%d' % (200,100))
        options.append('--start-node-id %d' % 20000000000)
        options.append('--start-way-id %d' % 10000000000)
        options.append('--max-nodes-per-tile %d' % 0)
        options.append('--hgtdir "%s"' % self.HGT)
        options.append('--pbf')

        cmdline = '%s %s' % ('phyghtmap', ' '.join(options))

        # print(cmdline)
        # ret = (None, None, None)
        ret = run(cmdline)

        for f in os.listdir(self.SRTM):
            f_old = os.path.abspath(os.path.join(self.SRTM, f))
            if f_old.startswith(unicode(fout)):
                f_new = os.path.abspath(os.path.join(self.SRTM, fout+'_srtm6.osm.pbf'))
                os.rename(f_old, f_new)
                break

        print('-----------------------------------------------')
        print('')

        if ret[0] != 0:
            fout = None

        return fout
        pass

    def download(self, binary=True):
        minlon = self.BBOX['min']['lon']
        minlat = self.BBOX['min']['lat']
        maxlon = self.BBOX['max']['lon']
        maxlat = self.BBOX['max']['lat']

        if abs((maxlon-minlon)*(maxlat-minlat)) >= 10:
            fout = self.download_list(binary)
            return fout


        print('Downloading %s.osm from overpass-api.de' % self.MAP)
        print('===============================================')

        if not os.path.exists(self.TARGET_FOLDER):
            os.makedirs(self.TARGET_FOLDER)
        fout = os.path.join(self.TARGET_FOLDER, self.MAP+'.osm')
        # print(fout)

        url = 'http://www.overpass-api.de/api/xapi_meta?*[bbox=%s]' % (self.BBOX0)

        # cmdline = 'wget -c -O "%s" "%s"' % (fout, url)
        cmdline = 'wget -x -O "%s" "%s"' % (fout, url)

        # print(cmdline)
        # os.system(cmdline)
        ret = run(cmdline)

        if binary:
            fout = self.convert(fout)
        print('-----------------------------------------------')
        print('')

        if ret[0] != 0:
            fout = None

        return fout
        pass

    def tagtransfer(self, fin, binary=True):
        fout = None
        if fin and os.path.isfile(fin):
            if binary:
                exto = '.pbf'
            else:
                exto = '.osm'
            fout = os.path.join(self.TARGET_FOLDER, self.MAP+'_taged'+exto)

            print('Tag Transfer %s to %s' % (fin, fout))
            print('===============================================')

            options = []
            ext = os.path.splitext(fin)[1].lower()
            if ext == '.pbf':
                options.append('--rb "%s"' % fin)
            elif ext == '.osm':
                options.append('--rx "%s"' % fin)
            options.append('--tag-transform file="%s"' % os.path.join(CWD, 'Tag-Transform.xml'))
            if binary:
                options.append('--wb "%s"' % fout)
            else:
                options.append('--wx "%s"' % fout)

            cmdline = '%s %s' % (self.OSMOSIS, ' '.join(options))

            # print(cmdline)
            # os.system(cmdline)
            ret = run(cmdline)

            print('-----------------------------------------------')
            print('')

            if ret[0] != 0:
                fout = None
        else:
            print('%s not exists!' % fin)

        return(fout)
        pass

    def cut(self, fin, binary=True):
        fout = None
        if fin and os.path.isfile(fin):
            if binary:
                exto = '.pbf'
            else:
                exto = '.osm'
            fout = os.path.join(self.TARGET_FOLDER, self.MAP+'_cutted'+exto)

            print('Cutting %s to %s' % (fin, fout))
            print('===============================================')

            options = []
            ext = os.path.splitext(fin)[1].lower()
            if ext == '.pbf':
                options.append('--rb "%s"' % fin)
            elif ext == '.osm':
                options.append('--rx "%s"' % fin)
            options.append('--tee %d' % 1)
            options.append('--bb %s' % self.BBOX0)
            if os.path.isfile(os.path.join(self.POLYGON, self.MAP+'.poly')):
                options.append('--bp file="%s"' % os.path.join(self.POLYGON, self.MAP+'.poly'))
            if binary:
                options.append('--wb "%s"' % fout)
            else:
                options.append('--wx "%s"' % fout)

            cmdline = '%s %s' % (self.OSMOSIS, ' '.join(options))

            # print(cmdline)
            # os.system(cmdline)
            ret = run(cmdline)

            print('-----------------------------------------------')
            print('')

            if ret[0] != 0:
                fout = None
        else:
            print('%s not exists!' % fin)

        return(fout)
        pass

    def sort(self, fin, binary=True):
        fout = None
        if fin and os.path.isfile(fin):
            if binary:
                exto = '.pbf'
            else:
                exto = '.osm'
            fout = os.path.join(self.TARGET_FOLDER, self.MAP+'_sorted'+exto)

            print('Sorting %s to %s' % (fin, fout))
            print('===============================================')

            options = []
            ext = os.path.splitext(fin)[1].lower()
            if ext == '.pbf':
                options.append('--rb "%s"' % fin)
            elif ext == '.osm':
                options.append('--rx "%s"' % fin)
            options.append('--sort')
            options.append('--bb %s' % self.BBOX0)
            if os.path.isfile(os.path.join(self.POLYGON, self.MAP+'.poly')):
                options.append('--bp file="%s"' % os.path.join(self.POLYGON, self.MAP+'.poly'))
            if binary:
                options.append('--wb "%s"' % fout)
            else:
                options.append('--wx "%s"' % fout)

            cmdline = '%s %s' % (self.OSMOSIS, ' '.join(options))

            # print(cmdline)
            # os.system(cmdline)
            ret = run(cmdline)

            print('-----------------------------------------------')
            print('')

            if ret[0] != 0:
                fout = None
        else:
            print('%s not exists!' % fin)

        return(fout)
        pass

    def merge(self, fin, binary=True):
        fout = None
        if fin and os.path.isfile(fin):
            if binary:
                exto = '.pbf'
            else:
                exto = '.osm'
            fout = os.path.join(self.TARGET_FOLDER, self.MAP+'_merged'+exto)

            print('Merging %s to %s' % (fin, fout))
            print('===============================================')

            options = []
            ext = os.path.splitext(fin)[1].lower()
            if ext == '.pbf':
                options.append('--rb "%s"' % fin)
            elif ext == '.osm':
                options.append('--rx "%s"' % fin)
            options.append('--rb %s_srtm6.osm.pbf' % os.path.join(self.SRTM, self.MAP))
            options.append('--merge')
            options.append('--tag-transform file=%s' % os.path.join(CWD, 'Tag-Transform.xml'))
            options.append('--bb %s' % self.BBOX0)
            if os.path.isfile(os.path.join(self.POLYGON, self.MAP+'.poly')):
                options.append('--bp file="%s"' % os.path.join(self.POLYGON, self.MAP+'.poly'))
            # options.append('--wx "%s"' % fout)
            if binary:
                options.append('--wb "%s"' % fout)
            else:
                options.append('--wx "%s"' % fout)

            cmdline = '%s %s' % (self.OSMOSIS, ' '.join(options))

            # print(cmdline)
            # os.system(cmdline)
            ret = run(cmdline)

            print('-----------------------------------------------')
            print('')

            if ret[0] != 0:
                fout = None
        else:
            print('%s not exists!' % fin)

        return(fout)
        pass

    def convert(self, fin):
        fout = None
        ext = os.path.splitext(fin)[1].lower()
        if ext == '.pbf':
            fout = fin
            return fout

        if fin and os.path.isfile(fin):
            fout = os.path.join(self.TARGET_FOLDER, self.MAP+'.pbf')

            print('Converting %s to %s' % (fin, fout))
            print('===============================================')

            options = []
            options.append('"%s"' % fin)
            options.append('-b=%s' % self.BBOX0)
            options.append('--out-pbf -o="%s"' % fout)

            cmdline = '%s %s' % (self.OSMCONVERT, ' '.join(options))

            # print(cmdline)
            # os.system(cmdline)
            ret = run(cmdline)

            print('-----------------------------------------------')
            print('')

            if ret[0] != 0:
                fout = None
        else:
            print('%s not exists!' % fin)

        return(fout)
        pass

    def makemap(self, fin, binary=True, hd=False, maptype=dict()):
        fout = None
        print(fin)
        if fin and os.path.isfile(fin):


            if ('mapforge' in maptype) and maptype['mapforge']:
                fout = os.path.join(self.TARGET_FOLDER, self.MAP+'.map')

                print('Making %s to %s' % (fin, fout))
                print('===============================================')

                options = []
                if binary:
                    options.append('--rb "%s"' % fin)
                else:
                    options.append('--rx "%s"' % fin)

                options.append('--mapfile-writer file="%s"' % fout)
                options.append('bbox=%s' % self.BBOX1)
                options.append('map-start-zoom=%d' % 12)
                # options.append('preferred-language=%s' % 'zh')
                options.append('preferred-language=%s' % self.LANG)
                if hd:
                    options.append('type=%s' % 'hd')
                options.append('tag-conf-file="%s"' % os.path.join(CWD, 'tag-mapping_oam.xml'))

                cmdline = '%s %s' % (self.OSMOSIS, ' '.join(options))

                # print(cmdline)
                # os.system(cmdline)
                ret = run(cmdline)

            if ('garmin' in maptype) and maptype['garmin']:
                fout = os.path.join(self.TARGET_FOLDER, self.MAP+'.img with more map file')

                print('Making %s to %s' % (fin, fout))
                print('===============================================')

                print('spliting large area......')

                options = []
                if binary:
                    options.append('--output=pbf')
                else:
                    options.append('--output=xml')
                options.append('--output-dir="%s"' % self.TARGET_FOLDER)
                options.append('--polygon-file="%s"' % os.path.join(self.POLYGON, self.MAP+'.poly'))
                options.append(fin)

                cmdline = '%s %s' % (self.SPLITTER, ' '.join(options))

                print(cmdline)
                # os.system(cmdline)
                ret = run(cmdline)


                print('making map......')
                options = []
                options.append('--read-config="%s.txt"' % self.MAP)

                # options.append('--input-file="%s"' % fin)
                if binary:
                    options.append(os.path.join(self.TARGET_FOLDER, '6324*.pbf'))
                else:
                    options.append(os.path.join(self.TARGET_FOLDER, '6324*.osm'))


                cmdline = '%s %s' % (self.MKGMAP, ' '.join(options))

                print(cmdline)
                # os.system(cmdline)
                ret = run(cmdline)

            print('-----------------------------------------------')
            print('')

            if ret[0] != 0:
                fout = None
        else:
            print('%s not exists!' % fin)

        return(fout)
        pass

    def MapMaking(self, binary=True):
        fout = None

        if self.MAP:
            if self.do_download:
                fout = self.download()
            else:
                if binary:
                    exto = '.pbf'
                else:
                    exto = '.osm'
                fout = os.path.join(self.TARGET_FOLDER, self.MAP+exto)

            if self.do_cut:
                fout = self.cut(fout, binary)

            if self.do_tag:
                fout = self.tagtransfer(fout, binary)

            if self.do_sort:
                fout = self.sort(fout, binary)

            if self.do_merge:
                fout = self.merge(fout, binary)

            if self.do_convert:
                fout = self.convert(fout)

            fout = self.makemap(fout, binary=self.do_convert, hd=self.do_hd, maptype=self.do_maptype)

            if fout:
                print('Making %s finished!' % fout)

        pass

    def test(self):
        fout = None

        self.download_list()

        pass


if __name__ == "__main__":
    print('MapForge OSM helper library')
