#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import codecs
# import urllib2
# import re

from bs4 import BeautifulSoup as Soup

from subprocess import Popen, PIPE
import threading

import lightmaps


CWD = os.path.abspath(os.path.dirname(__file__))
# JAVA_HOME = 'D:/Develop/java/jre7'
JAVA_HOME = os.path.join('D:', os.path.sep, 'Develop', 'java', 'jre7')


def run(cmd, options=None):
    if not 'JAVA_HOME' in os.environ:
        os.environ['JAVA_HOME'] = JAVA_HOME
    if not 'CLASSPATH' in os.environ:
        classpath = []
        classpath.append(os.path.join(JAVA_HOME, 'lib', 'dt.jar'))
        classpath.append(os.path.join(JAVA_HOME, 'lib', 'tools.jar'))
        os.environ['CLASSPATH'] = os.sep.join(classpath)

        binpath = []
        binpath.append(os.environ['PATH'])
        binpath.append(os.path.join(JAVA_HOME))
        binpath.append(os.path.join(JAVA_HOME, 'bin'))
        binpath.append(os.path.join(JAVA_HOME, 'jre', 'bin'))
        os.environ['PATH'] = os.path.pathsep.join(binpath)

    # print(os.environ['PATH'])

    try:
        if sys.stdout.isatty():
            console = True
    except:
        console = False

    if console:
        p = Popen(cmd, shell=True)
    else:
        # print(cmd)
        p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
        # p = Popen(cmd, shell=True, stdout=PIPE)

    if sys.platform in ('win32', 'win64', 'linux', 'linux2'):
        try:
            import affinity

            pa = affinity.get_process_affinity_mask(p.pid)
            if pa&1 == 0:
              pa_set = pa & 0x54 #0xaa
            else:
              pa_set = pa & 0xA8 #0x15
            # print(pa_set)
            affinity.set_process_affinity_mask(p.pid, pa_set)
            print('CPU affinity changed from 0x%x to 0x%x.' % (pa, affinity.get_process_affinity_mask(p.pid)))
        except:
            pass

        pass

    if console:
        stdout, stderr = p.communicate()
        return(p.returncode, None, None)
    else:
        stdout, stderr = p.communicate()
        if sys.platform in ('win32', 'win64'):
            coding = 'gbk'
        else:
            coding = sys.getdefaultencoding()
        print(stdout.decode(coding))
        print(stderr.decode(coding))
        return(p.returncode, stdout.decode(coding), stderr.decode(coding))
        # while 1:
        #     line = p.stdout.readline()
        #     if not line: break
        #     print(line)
            # line = p.stderr.readline()
            # print(line)
        # p.communicate()
        # return(p.returncode, None, None)

    # print 'return code : %s' % p.returncode # is 0 if success
    pass


def k2p_convert(fin, fout):
    if fin and fout:
        if os.path.exists(fin):
            with codecs.open(fin, 'r', encoding='utf8') as f:
                soup = Soup(f, ["lxml", "xml"])

                points_group = []

                for polygon in soup.find_all('Polygon'):
                    points = []
                    for node in polygon.find_all('coordinates'):
                       for coord in node.string.split():
                           point = coord.split(',')
                           points.append(point)
                       pass
                    points_group.append(points)


                with codecs.open(fout, 'w', encoding='utf8') as f:
                    lines = []

                    title = os.path.splitext(os.path.basename(fout))[0]
                    lines.append(title)

                    count = 0
                    for points in points_group:
                        count += 1
                        lines.append('%d' % (count))
                        for point in points:
                            line = format('   %e   %e' % (float(point[0]), float(point[1]))).upper()
                            lines.append(line)
                        lines.append('END')
                    lines.append('END')

                    f.write(os.linesep.join(lines))
            pass
        pass
    pass


def minimaps(center=(0,0), zoom=12):
    if lightmaps.X11:
        lightmaps.QtGui.QApplication.setGraphicsSystem('raster')

    app = lightmaps.QtGui.QApplication(sys.argv)
    app.setApplicationName('LightMaps')
    w = lightmaps.MapZoom()
    w.setWindowTitle("OpenStreetMap")

    if lightmaps.SYMBIAN or lightmaps.WINCE:
        w.showMaximized()
    else:
        w.resize(600, 450)

    w.map_.setCenter(center[0], center[1])
    w.map_._normalMap.zoom = zoom
    w.show()
    sys.exit(app.exec_())

    pass


if __name__ == "__main__":
    minimaps(center=(18.25, 109.5), zoom=14)
    print('OSM lib utils')


